/////////////////////////////////////////////////////////////////////////
////                                                                 ////
////                        ex_usb_common.h                          ////
////                                                                 ////
//// Common hardware definitions and configuration for all of CCS's  ////
//// USB example programs (ex_usb_*.c)                               ////
////                                                                 ////
//// One of the following configuration macros must be #defined:     ////
////  USB_HW_CCS_PIC16C765    CCS ACEKit with USB Addon and          ////
////                          PIC16C765.                             ////
////  USB_HW_CCS_USBN9604     CCS National USBN9604 USB development  ////
////                          kit (external USB peripheral).         ////
////  USB_HW_MCHP_18F14K50    Microchip low-pin count USB            ////
////                          development kit (PIC18F14K50).         ////
////  USB_HW_CCS_PIC18F4550   CCS PIC18F4550 USB Development kit     ////
////  USB_HW_MCHP_18F46J50    Microchip USB PIM Demo Board           ////
////                          (PIC18F46J50).                         ////
////  USB_HW_GENERIC_18F67J50 Generic 18F67J50 example.              ////
////  USB_HW_GENERIC_18F27J53 Generic 18F27J53 example.              ////
////  USB_HW_MCHP_EXPLORER16  Microchip Explorer16 with USB          ////
////                          OTG PICTail Plus.                      ////
////  USB_HW_CCS_PIC24F       CCS 24FJ256GB106 USB Development kit.  ////
////                                                                 ////
//// This file is part of CCS's PIC USB driver code.  See USB.H      ////
//// for more documentation and a list of examples.                  ////
////                                                                 ////
/////////////////////////////////////////////////////////////////////////
////                                                                 ////
//// VERSION HISTORY                                                 ////
////                                                                 ////
//// Oct 15th, 2010:                                                 ////
////  Added initial 18F47J53 family support.                         ////
////  Added USB_ISR_POLLING support.  Define this and interrupts     ////
////     will not be used.  usb_task() must be called periodically   ////
////     in your main loop.  If it is not called faster than once    ////
////     per millisecond, USB may not work (PIC18 and PIC24 only).   ////
////                                                                 ////
//// August 31st, 2010:                                              ////
////  Added USB_HW_MCHP_18F46J50, USB_HW_MCHP_18F14K50 and           ////
////  USB_HW_GENERIC_18F67J50 hardware.                              ////
////                                                                 ////
//// April 28th, 2010:                                               ////
////  Added ex_usb_common.h.                                         ////
////  Initial support for CCS PIC24USB board.                        ////
////  USB_CON_SENSE_PIN replaced with USB_CABLE_IS_ATTACHED()        ////
////     macro.  If USB_CON_SENSE_PIN is defined, it will create     ////
////     USB_CABLE_IS_ATTACHED() macro for you (for backwards        ////
////     compatibility).                                             ////
////                                                                 ////
/////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996,2010 Custom Computer Services         ////
//// This source code may only be used by licensed users of the CCS  ////
//// C compiler.  This source code may only be distributed to other  ////
//// licensed users of the CCS C compiler.  No other use,            ////
//// reproduction or distribution is permitted without written       ////
//// permission.  Derivative programs created using this software    ////
//// in object code form are not restricted in any way.              ////
/////////////////////////////////////////////////////////////////////////

////// Begin User Configuration

#define USB_HW_CCS_PIC18F4550     //CCS PIC18F4550 USB Development kit

// Optional configuration.
// Defining USB_ISR_POLLING will have USB library not use ISRs.  Instead you
// must periodically call usb_task().
//#define USB_ISR_POLLING

////// End User Configuration

#ifndef __EX_USB_COMMON_H__
#define __EX_USB_COMMON_H__


#if defined(USB_HW_CCS_PIC18F4550)
   #include "18F4550.h"
   #fuses HSPLL,NOWDT,NOPROTECT,NOLVP,NODEBUG,USBDIV,PLL5,CPUDIV1,VREGEN
   #use delay(clock=48000000)
   
   //leds ordered from bottom to top
   #DEFINE LED1 PIN_A5  //green
   #define LED2 PIN_B4  //yellow
   #define LED3 PIN_B5  //red
   #define BUTTON_PRESSED() !input(PIN_A4)

   //see section below labeled USB_CABLE_IS_ATTACHED
//@@@ remove hardware sensing code   #define PIN_USB_SENSE   PIN_B2

   #define HW_ADC_CONFIG   ADC_CLOCK_INTERNAL
   #define HW_ADC_CHANNEL  0
   #define HW_ADC_PORTS    AN0
#endif





#if defined(__NO_UART__)
   #define uart_putc(c)
   #define uart_getc()  (0)
   #define uart_kbhit() (FALSE)
   #define uart_printf(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z) { }
#else
   #use rs232(baud=57600, UART1, errors)

   #define uart_putc(c) putc(c)
   #define uart_getc  getc
   #define uart_kbhit kbhit
   #define uart_printf  printf
#endif

/////////////////////////////////////////////////////////////////////////////
//
// Required macro: USB_CABLE_IS_ATTACHED()
//
// This macro provides configuration to the library to detect if a 
// USB cable is attached or not.  This is only relevant if the PIC is acting
// as a slave device.
//
// If you are using a USB connection sense method or pin, define this
// macro here.  If you are not using connection sense, comment out this line.  
// Without connection sense you will not know if the device gets disconnected.
//
// If you are using a PIC24 with the internal USB peripheral, you can connect
// the Vbus (5V) line from the USB to the Vbus pin on the PIC - you
// can then look at the SFR bit U1OTGSTAT.SESVD to determine if USB is
// connected.
//
// If you are not using the internal USB peripheral of a PIC24, you can connect 
// Vbus to a GPIO to detect connection sense.
//
// For both methods (connecting to Vbus pin or GPIO pin), a pull-down resistor
// (around 150K) and capacitor (1uF) should also be placed on the Vbus pin.
//
/////////////////////////////////////////////////////////////////////////////

// @@@ remove unused ifs #if defined(__PCD__)
   //#define USB_CABLE_IS_ATTACHED() getenv("SFR:SESVD")
//   #define USB_CABLE_IS_ATTACHED() bit_test(*0x484,3)
//#elif defined(PIN_USB_SENSE)
//   #define USB_CABLE_IS_ATTACHED() input(PIN_USB_SENSE)
//#endif

#ifndef LED_ON
#define LED_ON(x) output_low(x)
#endif

#ifndef LED_OFF
#define LED_OFF(x) output_high(x)
#endif

#ifndef HW_INIT
#define HW_INIT()
#endif

#ifndef __USB_PIC_PERIF__
#define __USB_PIC_PERIF__  1
#endif

#endif
