/*
 Some utilities from my old opsys for use with USB




*/

// declare opsys globals here
// CCS default is unsigned, but it pays to be explicit for the humans
int8 i;  // I use this over and over, with care about nesting
int8 ii; // iterator for interrupts (only) to use

unsigned int8 secdiv; // tens of milliseconds in this version (0-99)
unsigned int8 seconds; // the time since startup
unsigned int8 minutes;
unsigned int16 hours;
int1 secflag; // boolean set once a second by Timer1 interrupt
int1 neutflag; // boolean toggled by PC for alt counter display
///////////////////
unsigned int16 CountsHigh; // high part of timer0 total count
unsigned int16 Counts3High; // high part of timer3 total count

unsigned int32 CountTemp; // for subtraction
unsigned int32 LastTotal; // created at runtime as needed
unsigned int32 Last3Total; // same for other counter
unsigned int16 CurrentCount; //@@@ 65k counts per second - software limit
unsigned int32 CPM1;     // counts per minute based on current second
unsigned int32 CPM10;   // for the ten second cpm output
unsigned int16 OldCounts[10];  // array for previous counts
unsigned int16 Old3Counts[10];  // array for previous counts
unsigned int8 OCI;            // old counts index
unsigned int8 OC3I;            // old counts index
//////////////////
#define NO_ONESHOTS 8 // how many oneshots we'll handle
unsigned int8 oneshots[NO_ONESHOTS]; // the oneshots, globals
//////////////////////////////

char outbuf[64]; // where we hold what we're going to send to the host

///////////////////////////////////////////////////////////////////////////

void ClearTimeCounts()
// clears times and counts.  Used at init or a runtime reset from PC
{
secdiv = seconds = minutes = OCI = OC3I = 0; // runtime clock, secdivs are 10 ms
CountsHigh = Counts3High = hours = 0; //grouped by size of integer
secflag = FALSE; // gets set once a second by timer1 isr.  A bool
LastTotal = Last3Total = 0; // init 32 bit integer
for (i=0;i<10;i++) OldCounts[i] = Old3Counts[i] = 0; // zero out counts history
for (i=0;i<NO_ONESHOTS;i++) oneshots[i] = 0; // zero the oneshots
set_timer0(0); // clear out timer hardware too
set_timer3(0); // both of em
}

////////////////////////////////////////////////////////////////////////////
void opsysinit()
{ // set things up

 ClearTimeCounts();  // clear timer and counters
 neutflag = FALSE;  //@@@ default is off, change if needed
/*************** Hardware init *************************/

// timer one counts the cpu system clock, to interrupt at 100hz (10ms)
// this is what we use for runtime and oneshot timer clock
setup_timer_1(T1_INTERNAL | T1_DIV_BY_8); // 1.5 mhz with 12 mhz cycle rate
set_timer1(65536-15000); // set to overflow at 100 hz
enable_interrupts(INT_TIMER1);

// timer 0 is the (first) counter we're counting -- geiger pancake
setup_timer_0(T0_EXT_H_TO_L | T0_DIV_1); // divides by two unless specified
set_timer0(0); // clear the timer
set_tris_a(get_tris_a() | 0x10); // make sure it's an input
enable_interrupts(INT_TIMER0);

// added later, a second counter we'll count inputs on -- optional
setup_timer_3 (T3_EXTERNAL | T3_DIV_BY_1);
set_timer3(0);
set_tris_c(get_tris_c() | 0x01); // make an input pin for sure (dox unclear)
enable_interrupts(INT_TIMER3); // make it live

// their code enables interrupts "somewhere" -- but make sure
enable_interrupts(GLOBAL); // turn on all enabled interrupts
}
/////////////////////////////////////////////////////////////////////////////

// subroutines

///////////////////////////////////////
// Call this once a second from the main loop.  Clears the seconds flag and
// does the mundane conversions for time in seconds to clock time (since start)
// Timer1 interrupt sets secflag for background to know when to call us
void TimeMachine()
{
 if (++seconds >= 60) // the use of >= makes this work even without good init
 {
  seconds = 0; // new minute
  if (++minutes >= 60) // >= belt AND suspenders
  {
   minutes = 0; // new hour
   hours++; // so do that increment too
  } // end if minutes overflow
 }// end if seconds overflow
// we just hope hours don't overflow in a 16 bit int -- that's a long run
 secflag = FALSE; // clear seconds flag, set by timer1 interrupt
  
 // print first part of output line - both serial and USB output
 sprintf(outbuf,"\n%Ld:%02u:%02u ",hours,minutes,seconds); // trailing space
 uart_printf("%s",outbuf); // side benefit - blocking call lets USB get ready
 usb_cdc_puts(outbuf); // for this, so we're using uart as a delay...
}
///////////////////////////////////////
void DoCounts()
{ // called once per second (we hope!)
 CountTemp = make32(CountsHigh,get_timer0()); // get total counts
 CurrentCount = CountTemp - LastTotal; // counts for this second
//@@@ if the above returns a negative number, we should fix the rollover error
// and maybe have to do this in all 32 bit to even tell....

 CPM1 = CurrentCount * 60;  // output value for CPM based on recent second
 OldCounts[OCI] = CurrentCount; // store for later sum of 10
 CPM10 = 0; // start this fresh each time
 for (i=0;i<10;i++) CPM10 += OldCounts[i]; // sum last 10 counts
 CPM10 *= 6; // make this into counts/minute too
 if (++OCI > 9) OCI = 0;  // roll index over at the right time

//output to uart (used as delay/debug) and USB both here
 sprintf(outbuf,"CPM1 %Lu CPM10 %Lu TC %Lu",CPM1,CPM10,CountTemp);
 uart_printf("%s",outbuf); //output to serial port
 usb_cdc_puts(outbuf); // for this, so we're using uart as a delay...
 LastTotal = CountTemp; // keep up to speed on this
  
 // now the code for the other counter, which can share some of the variables
 CountTemp = make32(Counts3High,get_timer3()); // get total for C3
 CurrentCount = CountTemp - Last3Total; //same as above, but one unique var
 CPM1 = CurrentCount * 60;
 Old3Counts[OC3I] = CurrentCount; // update 10 sec buffer
 CPM10 = 0;
 for (i=0;i<10;i++) CPM10 += Old3Counts[i];
 CPM10 *= 6;
 if (++OC3I > 9) OC3I = 0;
 if (neutflag)
 { // only print if the flag is true, but compute regardless
  sprintf(outbuf," NPM1 %Lu NPM10 %Lu TNC %Lu",CPM1,CPM10,CountTemp);
  uart_printf("%s",outbuf); // serial port and delay
  usb_cdc_puts(outbuf); // to USB too
 }
 Last3Total = CountTemp; // keep track of last raw total number
}

///////////////////////////////////////
///////////////////////////////////////








/******************Interrupts*************************************************/
///////////////////////////////////////
#INT_TIMER1
void intTimer1()
{
 set_timer1(65536-15000); // get ready for next time, count for 10ms
 secdiv++; // count 10 ms times till a second's worth
 if (secdiv >= 100)
  {
   secdiv = 0; // ready for next second's counting
   secflag = TRUE; // tell background about the new second
  }
// oneshots code here...count down any non-zero oneshots till they are zero
// at 10 ms per count.  You can set these in the background, then check for
// done once per main loop, and your timing delay is what you set.  This way
// you don't have to sit in a spin loop and wait, so other things can be
// run while a oneshot counts down
 for (ii=0;ii<NO_ONESHOTS;ii++)
 {
  if (oneshots[ii]) oneshots[ii]--; // count down any non-zero ones
 }
}
//////////////////////////////////////////
//  This counts another 16 bits when timer0 overflows for 32 bits total
//  low part of count is timer0 itself
#INT_TIMER0
void intTimer0()
{ // not a real complex ISR, just count the high 16 bits on overflow
 ++CountsHigh; // keep track of overflows
}
/////////////////////////////////////////////////////////////////
#INT_TIMER3
void intTimer3 ()
{ // count alternate input pin too
 ++Counts3High; // same as above for another counter
}
